/**
 * 
 */
package sync.smartapp;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.Vector;

/**
 * @author naveenb
 * 
 */
public class SyncDAO {

	private static String url = "jdbc:derby:net://localhost:1527/syncDB:user=syncadmin;password=syncadmin;retrieveMessagesFromServerOnGetMessage=true;";

	private static String INSERT_ORDER_SQL = "insert into order_info(orderid,orderdate,userid,customerid,productname,productquantity,orderstatus,syncstatus,updatedate) values (?,?,?,?,?,?,?,?,?)";

	private static String SELECT_SYNC_ORDER_SQL = "select * from order_info where userid = ? and syncstatus = ?";

	private Timestamp ts = new java.sql.Timestamp(System.currentTimeMillis());

	static {
		try {
			// db2jcc.jar;db2jcc_license_c.jar
			Class.forName("com.ibm.db2.jcc.DB2Driver");
		} catch (Exception e) {
			System.out.println("unable to load driver " + e.getMessage());
		}
	}

	public String performProcess(Vector vector) throws Exception {

		String data = null;
		Vector responseVect = new Vector();
		if (vector != null && vector.size() > 1) {
			insertSyncData(vector);
		}
		responseVect = retreiveSyncData(vector);

		data = formulateXML(responseVect);
		return data;

	}

	/**
	 * @param vect
	 * @throws Exception
	 */
	private void insertSyncData(Vector vect) throws Exception {

		Connection con = DriverManager.getConnection(url);
		PreparedStatement pstmt = null;

		int vsize = 0;

		vsize = vect.size();
		try{

		// Remove first element as it contains user object

		for (int i = 1; i < vsize; i++) {

			OrderBean orderBean = (OrderBean) vect.get(i);
			pstmt = con.prepareStatement(INSERT_ORDER_SQL);
			pstmt.setString(1, orderBean.getOrderId());
			pstmt.setTimestamp(2, new Timestamp(Long.valueOf(
					orderBean.getOrderDate()).longValue()));
			pstmt.setString(3, orderBean.getUserId());
			pstmt.setString(4, orderBean.getCustomerId());
			pstmt.setString(5, orderBean.getProductName());
			pstmt.setString(6, orderBean.getProductQuantity());
			pstmt.setString(7, orderBean.getOrderStatus());
			// Set syncststus to "Y"
			pstmt.setString(8, "Y");
			System.out.print(ts);
			pstmt.setTimestamp(9, ts);

			int count = pstmt.executeUpdate();
		}
		}
		finally{
			if(pstmt!=null){
				pstmt.close();
			}
			if(con!=null){
				con.close();
			}
		}
		
	}

	/**
	 * @param vect
	 * @return
	 * @throws Exception
	 */
	private Vector retreiveSyncData(Vector vect) throws Exception {

		Vector responseVect = new Vector();
		String userId = (String) vect.get(0);

		Connection con = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			con = DriverManager.getConnection(url);
			pstmt = con.prepareStatement(SELECT_SYNC_ORDER_SQL);
			pstmt.setString(1, userId);
			pstmt.setString(2, "Y");

			rs = pstmt.executeQuery();

			while (rs.next()) {
				OrderBean orderBean = new OrderBean();
				orderBean.setOrderId(rs.getString(2));

				java.util.Date dt = (java.util.Date) rs.getTimestamp(3);
				// java.util.Date df =
				// DateFormat.getDateTimeInstance().parse(String.valueOf(rs.getTimestamp(3)));
				orderBean.setOrderDate(String.valueOf(dt.getTime()));
				orderBean.setUserId(rs.getString(4));
				orderBean.setCustomerId(rs.getString(5));
				orderBean.setProductName(rs.getString(6));
				orderBean.setProductQuantity(rs.getString(7));
				orderBean.setOrderStatus(rs.getString(8));
				orderBean.setSyncStatus(rs.getString(9));
				responseVect.add(orderBean);
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
			if (con != null) {
				con.close();
			}

		}

		return responseVect;
	}

	/**
	 * @param vect
	 * @return
	 */
	private String formulateXML(Vector vect) {
		StringBuffer sb = new StringBuffer();
		int len = 0;

		if (vect != null) {
			len = vect.size();
		}
		sb.append("<sync-data>");
		sb.append("<userid>");
		sb.append("</userid>");

		if (len > 0) {

			for (int k = 0; k < len; k++) {

				OrderBean orderBean = (OrderBean) vect.elementAt(k);
				sb.append("<record>");

				sb.append("<id>");
				sb.append(orderBean.getOrderId());
				sb.append("</id>");

				sb.append("<orderdate>");
				sb.append(orderBean.getOrderDate());
				sb.append("</orderdate>");

				sb.append("<userid>");
				sb.append(orderBean.getUserId());
				sb.append("</userid>");

				sb.append("<customerid>");
				sb.append(orderBean.getCustomerId());
				sb.append("</customerid>");

				sb.append("<productname>");
				sb.append(orderBean.getProductName());
				sb.append("</productname>");

				sb.append("<productquantity>");
				sb.append(orderBean.getProductQuantity());
				sb.append("</productquantity>");

				sb.append("<orderstatus>");
				sb.append(orderBean.getOrderStatus());
				sb.append("</orderstatus>");

				sb.append("<syncstaus>");
				sb.append(orderBean.getSyncStatus());
				sb.append("</syncstaus>");

				sb.append("</record>");
			}
		}
		sb.append("</sync-data>");

		return sb.toString();

	}

}
